# coding: utf-8
from collections import deque
from pathlib import Path

from clonus.methods.method_configurator import VariablesCompare


def merged(points, error=5):
    try:
        merged_points = [points[0]]
        for i in range(1, len(points)):
            last = merged_points[-1]
            if last[0] <= points[i][0] <= last[1]:
                if points[i][1] > last[1]:
                    merged_points = merged_points[:-1]
                    merged_points.append([last[0], points[i][1]])
            else:
                if points[i][0] - last[1] <= error:
                    merged_points = merged_points[:-1]
                    merged_points.append([last[0], points[i][1]])
                else:
                    merged_points.append(points[i])
        return merged_points
    except IndexError:
        return None


def generate_str_code_from_file(fil: Path):
    contents = ""
    with open(fil, "r", encoding="utf-8") as f:
        for line in f:
            contents += line
    return contents


def create_fragments(var_compare: VariablesCompare):
    if len(var_compare.clone_part_1) != 0 and len(var_compare.clone_part_2) != 0:
        res_list = dict()
        marked_str1, new_clone_part_1 = fix_1(var_compare.file1, var_compare.clone_part_1, 1, var_compare.path2)
        marked_str2, new_clone_part_2 = fix_1(var_compare.file2, var_compare.clone_part_2, 0, var_compare.path1)
        # alt_marked_str1 = fill(var_compare.file1, var_compare.clone_part_1, 1, var_compare.path2)
        # alt_marked_str2 = fill(var_compare.file2, var_compare.clone_part_2, 0, var_compare.path1) <-- альтернативный способ вывода текста в отчете (не построчно)
        res_list[var_compare.f1] = marked_str1
        res_list[var_compare.f2] = marked_str2
        return res_list


def fill(str_code, clone_part, number_file, path):
    copy_lines = []
    length = len(clone_part)
    prev_r_b = 0
    for i in range(length):
        l_b, r_b = clone_part[i]
        if not (len(str_code[prev_r_b:l_b]) == 0 and i == 0):
            copy_lines.append({'text': str_code[prev_r_b:l_b], 'mark': False})
        copy_lines.append({'text': str_code[l_b:r_b + 1], 'mark': True})
        prev_r_b = r_b + 1
        if i == length - 1 and len(str_code[r_b + 1:]) != 0:
            copy_lines.append({'text': str_code[r_b + 1:], 'mark': False})
    return concat(copy_lines, number_file, path)


def fix_1(str_code: str, clone_part, number_file, path):
    arr_str = splitn(str_code)
    copy_lines = []
    method_res = []
    cur = 0
    for p in clone_part:
        left, right = p[0], p[1]
        if len(method_res) != 0 and method_res[-1][1] >= left:
            left = method_res[-1][1] + 1
        st = next(iter(filter(lambda x: x['start'] <= left, reversed(arr_str))), None)
        if st is None:
            st = next(iter(filter(lambda x: x['start'] > left, arr_str)), None)
        if st['end'] >= right:
            en = st
        else:
            en = next(iter(filter(lambda x: x['end'] >= right, arr_str)), None)
        filtered_arr = list(filter(lambda x: st['number'] <= x['number'] <= en['number'], arr_str))
        left, right = st['start'], en['end']
        method_res.append([left, right])
        if cur - left != 0:
            copy_lines.append({'text': str_code[cur:left], 'mark': False})
        cur = right + 1
        copy_lines.append({'text': "".join([d['string'] for d in filtered_arr]), 'mark': True})
    if cur < len(str_code):
        copy_lines.append({'text': str_code[cur:], 'mark': False})
    return concat(copy_lines, number_file, path), merged(method_res)


def splitn(star: str):
    lst_str = [{}]
    lst_str.remove(lst_str[0])
    ind = star.find('\n')
    start_pos = 0
    number = 1
    try:
        while ind < len(star) and ind != -1:
            while ind != -1:
                next_ind = star.find("\n", ind + 1)
                if next_ind == -1 or next_ind - ind != 1:
                    break
                ind = next_ind
            lst_str.append({'number': number, 'string': star[start_pos:ind + 1], 'start': start_pos, 'end': ind})
            start_pos = ind + 1
            ind = star.find('\n', start_pos)
            number += 1
    except Exception as ex:
        print(f'{ex.__context__} ind= {ind}')
    return lst_str


def concat(copy_lines, number_file, path):
    copy_lines_2 = combo(copy_lines)
    res_str = "<pre>"
    i = 0
    for c in copy_lines_2:
        if c['mark']:
            i += 1
            res_str += f'<a name="{i}"></a><a href="{path}#{i}" target="{number_file}"><object>{i}-></object></a><span style="color:red">{c['text']}</span>'
        else:
            res_str += c['text']
    res_str += "</pre>"
    return res_str


def combo(copy_lines):
    prev = None
    s = ""
    combo_lst = []
    for c in copy_lines:
        if prev is not None and prev == c['mark']:
            s += c['text']
        elif prev is not None and prev != c['mark']:
            combo_lst.append({'text': s, 'mark': prev})
            prev = c['mark']
            s = "".join(c['text'])
        elif prev is None:
            prev = c['mark']
            s = "".join(c['text'])
    if not s.__eq__(""):
        combo_lst.append({'text': s, 'mark': prev})
    return combo_lst
