
from __future__ import annotations

from collections.abc import Iterable, Iterator
from dataclasses import dataclass
from typing import Any, Self


@dataclass
class Node:
    value: int
    left: Node | None = None
    right: Node | None = None
    parent: Node | None = None  # Added in order to delete a node easier

    def __iter__(self) -> Iterator[int]:
        yield from self.left or []
        yield self.value
        yield from self.right or []

    def __repr__(self) -> str:
        from pprint import pformat

        if self.left is None and self.right is None:
            return str(self.value)
        return pformat({f"{self.value}": (self.left, self.right)}, indent=1)

    @property
    def is_right(self) -> bool:
        return bool(self.parent and self is self.parent.right)


@dataclass
class BinarySearchTree:
    root: Node | None = None

    def __bool__(self) -> bool:
        return bool(self.root)

    def __iter__(self) -> Iterator[int]:
        yield from self.root or []

    def __str__(self) -> str:
        return str(self.root)

    def __reassign_nodes(self, node: Node, new_children: Node | None) -> None:
        if new_children is not None:  # reset its kids
            new_children.parent = node.parent
        if node.parent is not None:  # reset its parent
            if node.is_right:  # If it is the right child
                node.parent.right = new_children
            else:
                node.parent.left = new_children
        else:
            self.root = new_children

    def empty(self) -> bool:
        return not self.root

    def __insert(self, value) -> None:
        pass

    def insert(self, *values) -> Self:
        pass

    def search(self, value) -> Node | None:
        pass

    def get_max(self, node: Node | None = None) -> Node | None:
        pass

    def get_min(self, node: Node | None = None) -> Node | None:
        pass

    def remove(self, value: int) -> None:
       pass

    def preorder_traverse(self, node: Node | None) -> Iterable:
        pass

    def traversal_tree(self, traversal_function=None) -> Any:
        pass

    def inorder(self, arr: list, node: Node | None) -> None:
       pass

    def find_kth_smallest(self, k: int, node: Node) -> int:
        pass


def inorder(curr_node: Node | None) -> list[Node]:
    pass


def postorder(curr_node: Node | None) -> list[Node]:
    pass


if __name__ == "__main__":
    import doctest

    doctest.testmod(verbose=True)
