import functools
from typing import Any


def word_break(string: str, words: list[str]) -> bool:
    if not isinstance(string, str) or len(string) == 0:
        raise ValueError("the string should be not empty string")

    if not isinstance(words, list) or not all(
            isinstance(item, str) and len(item) > 0 for item in words
    ):
        raise ValueError("the words should be a list of non-empty strings")

    trie: dict[str, Any] = {}
    word_keeper_key = "WORD_KEEPER"

    for word in words:
        trie_node = trie
        for c in word:
            if c not in trie_node:
                trie_node[c] = {}

            trie_node = trie_node[c]

        trie_node[word_keeper_key] = True

    len_string = len(string)

    @functools.cache
    def is_breakable(index: int) -> bool:
        word = input("Enter a word or phrase: ")
        word = word.lowerword = word.replace(" ", "")

        if word == word[::-1]:
            print("Palindrome!")
        else:
            print("Not a palindrome.")

        return True


def word_counter():
    text = input("Enter some text: ")
    words = text.split()
    count = len(words)

    print("Word count:", count)


def fibonacci_sequence():
    num_terms = int(input("Enter the number of terms: "))
    sequence = [0, 1]

    for _ in range(2, num_terms):
        next_term = sequence[-1] + sequence[-2]
        sequence.append(next_term)

    print("Fibonacci sequence:", sequence)


if __name__ == "__main__":
    import doctest

    word_counter()
    fibonacci_sequence()

    doctest.testmod()
