from django.contrib.sessions.backends.db import SessionStore
from django.core.handlers.wsgi import WSGIRequest
from django.forms import model_to_dict
from django.shortcuts import render, redirect
from django.http import HttpRequest
from pathlib import Path
from collections import deque
from itertools import combinations
from more_itertools import ilen
from rest_framework.response import Response

from clonus.forms import FileToFileForm, ManyFilesForm
from clonus.methods.counting_time import time_compare
from clonus.methods.fragments import create_fragments, generate_str_code_from_file
from clonus.methods.main import main
from clonus.models import Package, MultiPackage

from clonus.methods.fp_method_builder import FingerprintMethodBuilder
from clonus.methods.method_configurator import MethodConfigurator, VariablesCompare
from rest_framework.views import APIView

from clonus.serializers import PackageSerialiser
from django.views.decorators.clickjacking import xframe_options_exempt


# Create your views here.
class SummaryAPIView(APIView):

    def post(self, request):
        dict_data = {'gram_size': request.data['gram_size'], 'window_size': request.data['window_size']}
        serialiser = PackageSerialiser(data=dict_data)
        serialiser.is_valid(raise_exception=True)

        obj_package = serialiser.save()
        obj_package.mkdir()

        files = request.FILES
        i = 0
        for file in files:
            f = files[file]
            i += 1
            handle_uploaded_file(f, obj_package.path, i)

        obj_package.file1 = f'{obj_package.path}\\[1]{request.FILES["file1"].name}'
        obj_package.file2 = f'{obj_package.path}\\[2]{request.FILES["file2"].name}'
        obj_package.file_template = f'{obj_package.path}\\[3]{request.FILES["file_template"].name}'
        obj_package.gen_hash()
        obj_package.save()

        filenames = [obj_package.file1, obj_package.file2, obj_package.file_template]
        fp_builder = FingerprintMethodBuilder(filenames, obj_package.gram_size, obj_package.window_size)
        config = MethodConfigurator(fp_builder)
        method_res = config.make_method()
        obj_package.coeff = method_res.clone_pct
        obj_package.processed = True
        obj_package.save()

        context = {
            "hash": obj_package.hash,
            "coeff": obj_package.coeff * 100,
            "file1": str(obj_package.file1),
            "file2": str(obj_package.file2),
            "file_template": str(obj_package.file_template),
            "f1": Path(obj_package.file1).name,
            "f2": Path(obj_package.file2).name,
            "date_compare": obj_package.date,
            "q1": method_res.clone_parts_1,
            "q2": method_res.clone_parts_2,

        }
        return Response({"res": context})


def handle_uploaded_file(f, path, i):
    with open(f'{path}\\[{i}]{f.name}', "wb+") as destination:
        for chunk in f.chunks():
            destination.write(chunk)


def index(request: HttpRequest):
    #main()
    #time_compare(f'http://{request.get_host()}')
    return render(request, "index.html")


def compare(request: HttpRequest):
    if request.method == "POST":
        form = FileToFileForm(request.POST, request.FILES)
        if form.is_valid():
            p = Package(
                gram_size=request.POST["gram_size"],
                window_size=request.POST["window_size"],
            )
            p.save()
            p.mkdir()
            number = 1
            for file in (request.FILES["file1"], request.FILES["file2"], request.FILES["file_template"]):
                with open(f'{p.path}\\[{number}]{file.name}', "wb+") as f:
                    for chunk in file.chunks():
                        f.write(chunk)
                number += 1
            p.file1 = f'{p.path}\\[1]{request.FILES["file1"].name}'
            p.file2 = f'{p.path}\\[2]{request.FILES["file2"].name}'
            p.file_template = f'{p.path}\\[3]{request.FILES["file_template"].name}'
            p.gen_hash()
            p.save()
            return redirect("summary", h=p.hash)

    else:
        form = FileToFileForm()
    context = {"form": form}
    return render(request, "send.html", context)


def compare_many(request: HttpRequest):
    if request.method == "POST":
        form = ManyFilesForm(request.POST, request.FILES)
        if form.is_valid():
            p = MultiPackage(
                gram_size=request.POST["gram_size"],
                window_size=request.POST["window_size"],
            )
            p.save()
            p.mkdir()
            for file in request.FILES.getlist("files"):
                with open(p.path / file.name, "wb+") as f:
                    for chunk in file.chunks():
                        f.write(chunk)
            p.files = p.path
            p.gen_hash()
            p.save()
            return redirect("summary_many", h=p.hash)

    else:
        form = ManyFilesForm()
    context = {"form": form}
    return render(request, "send_many.html", context)


@xframe_options_exempt
def frame1(request, h: str):
    return render(request, "frame1.html", context=request.session.get('content'))


@xframe_options_exempt
def frame2(request, h: str):
    return render(request, "frame2.html", context=request.session.get('content'))


def summary(request, h: str):
    try:
        p = Package.objects.get(hash=h)
    except Package.DoesNotExist:
        return redirect("index")

    filenames = [p.file1, p.file2, p.file_template]
    fp_builder = FingerprintMethodBuilder(filenames, p.gram_size, p.window_size)
    config = MethodConfigurator(fp_builder)
    method_res = config.make_method()
    p.coeff = method_res.clone_pct
    p.processed = True
    p.save()
    file1 = generate_str_code_from_file(p.file1)
    file2 = generate_str_code_from_file(p.file2)
    f1 = Path(p.file1).name
    f2 = Path(p.file2).name
    host = 'http://127.0.0.1:8000'
    path1 = f'{host}{request.path}/frame1/'
    path2 = f'{host}{request.path}/frame2/'
    var = VariablesCompare(file1, file2, method_res.clone_parts_1, method_res.clone_parts_2, f1, f2, path1, path2)
    all_fragments = create_fragments(var)

    context = {
        "hash": p.hash,
        "coeff": p.coeff * 100,
        "file1": file1,
        "file2": file2,
        "f1": f1,
        "f2": f2,
        "fragments1": all_fragments[f1],
        "fragments2": all_fragments[f2],
        "path_frame1": path1,
        "path_frame2": path2,
    }

    request.session['content'] = context
    return render(request, "summary.html", context)


def summary_many(request: HttpRequest, h: str):
    try:
        p = MultiPackage.objects.get(hash=h)
    except MultiPackage.DoesNotExist:
        return redirect("index")

    contents = []
    names = []

    for file in Path(p.path).iterdir():
        with open(file, "r", encoding="utf8") as f:
            names.append(file.name)
            contents.append(f.read())

    rep = []
    for f1, f2 in combinations(Path(p.path).iterdir(), 2):
        fp_builder = FingerprintMethodBuilder([f1, f2], p.gram_size, p.window_size)
        config = MethodConfigurator(fp_builder)
        method_res = config.make_method()
        # coeffs.append(method_res.clone_pct * 100)
        p.processed = True
        p.save()
        # names1.append(f1.name)
        # names2.append(f2.name)
        rep.append([method_res.clone_pct * 100, f1.name, f2.name])

    rep.sort(key=lambda x: x[0], reverse=True)

    coeffs = [r[0] for r in rep]
    names1 = [r[1] for r in rep]
    names2 = [r[2] for r in rep]

    context = {
        "hash": p.hash,
        "data1": zip(names, contents),
        "data2": zip(coeffs, names1, names2),
    }
    return render(request, "summary_many.html", context)


def list(request: HttpRequest):
    q1 = Package.objects.all().order_by("-date")
    q2 = MultiPackage.objects.all().order_by("-date")
    context = {
        "packages": ((i, Path(i.file1).name, Path(i.file2).name) for i in q1),
        "multipackages": (
            (i, f"Количество файлов: {ilen(Path(i.path).iterdir())}") for i in q2
        ),
    }
    return render(request, "list.html", context)
